@@ -19,6 +19,8 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " % d   e r r o r s ,   % d   w a r n i n g " ]   =   " % 1 $ d   e r r o r s ,   % 2 $ d   w a r n i n g " ; 
  l o c a l i z e d S t r i n g s [ " % d   e r r o r s ,   % d   w a r n i n g s " ]   =   " % 1 $ d   e r r o r s ,   % 2 $ d   w a r n i n g s " ; 
  l o c a l i z e d S t r i n g s [ " % d   m a t c h e s " ]   =   " % d   m a t c h e s " ; 
+ l o c a l i z e d S t r i n g s [ " % d   s t y l e   c h a n g e " ]   =   " % d   s t y l e   c h a n g e " ; 
+ l o c a l i z e d S t r i n g s [ " % d   s t y l e   c h a n g e s " ]   =   " % d   s t y l e   c h a n g e s " ; 
  l o c a l i z e d S t r i n g s [ " % d   w a r n i n g " ]   =   " % d   w a r n i n g " ; 
  l o c a l i z e d S t r i n g s [ " % d   w a r n i n g s " ]   =   " % d   w a r n i n g s " ; 
  l o c a l i z e d S t r i n g s [ " % d      % d " ]   =   " % 1 $ d      % 2 $ d " ; 
@@ -32,22 +34,23 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " ( t e x t ) " ]   =   " ( t e x t ) " ; 
  l o c a l i z e d S t r i n g s [ " ( w h i t e s p a c e ) " ]   =   " ( w h i t e s p a c e ) " ; 
  l o c a l i z e d S t r i n g s [ " 1   m a t c h " ]   =   " 1   m a t c h " ; 
+ l o c a l i z e d S t r i n g s [ " A l l " ]   =   " A l l " ; 
  l o c a l i z e d S t r i n g s [ " A l w a y s   e n a b l e " ]   =   " A l w a y s   e n a b l e " ; 
  l o c a l i z e d S t r i n g s [ " A n   e r r o r   o c c u r r e d   t r y i n g   t o \ n r e a d   t h e    % s    t a b l e . " ]   =   " A n   e r r o r   o c c u r r e d   t r y i n g   t o \ n r e a d   t h e    % s    t a b l e . " ; 
  l o c a l i z e d S t r i n g s [ " A n   u n e x p e c t e d   e r r o r   % s   o c c u r r e d . " ]   =   " A n   u n e x p e c t e d   e r r o r   % s   o c c u r r e d . " ; 
  l o c a l i z e d S t r i n g s [ " A v e r a g e " ]   =   " A v e r a g e " ; 
  l o c a l i z e d S t r i n g s [ " B r e a k p o i n t s " ]   =   " B r e a k p o i n t s " ; 
+ l o c a l i z e d S t r i n g s [ " C O O K I E S " ]   =   " C O O K I E S " ; 
  l o c a l i z e d S t r i n g s [ " C a l l   S t a c k " ]   =   " C a l l   S t a c k " ; 
  l o c a l i z e d S t r i n g s [ " C a l l s " ]   =   " C a l l s " ; 
+ l o c a l i z e d S t r i n g s [ " C l e a r   c h a n g e s   l o g . " ]   =   " C l e a r   c h a n g e s   l o g . " ; 
  l o c a l i z e d S t r i n g s [ " C l e a r   c o n s o l e   l o g . " ]   =   " C l e a r   c o n s o l e   l o g . " ; 
  l o c a l i z e d S t r i n g s [ " C l o s u r e " ]   =   " C l o s u r e " ; 
- l o c a l i z e d S t r i n g s [ " C O O K I E S " ]   =   " C O O K I E S " ; 
  l o c a l i z e d S t r i n g s [ " C o m p u t e d   S t y l e " ]   =   " C o m p u t e d   S t y l e " ; 
  l o c a l i z e d S t r i n g s [ " C o o k i e s " ]   =   " C o o k i e s " ; 
  l o c a l i z e d S t r i n g s [ " D A T A B A S E S " ]   =   " D A T A B A S E S " ; 
  l o c a l i z e d S t r i n g s [ " D O M " ]   =   " D O M " ; 
  l o c a l i z e d S t r i n g s [ " D a t a b a s e   n o   l o n g e r   h a s   e x p e c t e d   v e r s i o n . " ]   =   " D a t a b a s e   n o   l o n g e r   h a s   e x p e c t e d   v e r s i o n . " ; 
- l o c a l i z e d S t r i n g s [ " D a t a b a s e s " ]   =   " D a t a b a s e s " ; 
  l o c a l i z e d S t r i n g s [ " D e b u g g i n g   d i s a b l e d .   C l i c k   t o   e n a b l e . " ]   =   " D e b u g g i n g   d i s a b l e d .   C l i c k   t o   e n a b l e . " ; 
  l o c a l i z e d S t r i n g s [ " D e b u g g i n g   e n a b l e d .   C l i c k   t o   d i s a b l e . " ]   =   " D e b u g g i n g   e n a b l e d .   C l i c k   t o   d i s a b l e . " ; 
  l o c a l i z e d S t r i n g s [ " D e b u g g i n g   s c r i p t s   r e q u i r e s   y o u   t o   s t a r t   t h e   d e b u g g e r . " ]   =   " D e b u g g i n g   s c r i p t s   r e q u i r e s   y o u   t o   s t a r t   t h e   d e b u g g e r . " ; 
@@ -55,7 +58,10 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " D i m e n s i o n s " ]   =   " D i m e n s i o n s " ; 
  l o c a l i z e d S t r i n g s [ " D o c k   t o   m a i n   w i n d o w . " ]   =   " D o c k   t o   m a i n   w i n d o w . " ; 
  l o c a l i z e d S t r i n g s [ " D o c u m e n t s " ]   =   " D o c u m e n t s " ; 
+ l o c a l i z e d S t r i n g s [ " D o m a i n " ]   =   " D o m a i n " ; 
  l o c a l i z e d S t r i n g s [ " D o n ' t   p a u s e   o n   e x c e p t i o n s . " ]   =   " D o n  t   p a u s e   o n   e x c e p t i o n s . " ; 
+ l o c a l i z e d S t r i n g s [ " D o u b l e - C l i c k   t o   A d d " ]   =   " D o u b l e - C l i c k   t o   A d d " ; 
+ l o c a l i z e d S t r i n g s [ " D o u b l e - C l i c k   t o   t o g g l e   b e t w e e n   U R L   e n c o d e d   a n d   d e c o d e d   f o r m a t s " ]   =   " D o u b l e - C l i c k   t o   t o g g l e   b e t w e e n   U R L   e n c o d e d   a n d   d e c o d e d   f o r m a t s " ; 
  l o c a l i z e d S t r i n g s [ " E l e m e n t s " ]   =   " E l e m e n t s " ; 
  l o c a l i z e d S t r i n g s [ " E n a b l e   D e b u g g i n g " ]   =   " E n a b l e   D e b u g g i n g " ; 
  l o c a l i z e d S t r i n g s [ " E n a b l e   P r o f i l i n g " ]   =   " E n a b l e   P r o f i l i n g " ; 
@@ -66,21 +72,26 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " E v e n t   D o c u m e n t " ]   =   " E v e n t   D o c u m e n t " ; 
  l o c a l i z e d S t r i n g s [ " E v e n t   T a r g e t " ]   =   " E v e n t   T a r g e t " ; 
  l o c a l i z e d S t r i n g s [ " E x c l u d e   s e l e c t e d   f u n c t i o n . " ]   =   " E x c l u d e   s e l e c t e d   f u n c t i o n . " ; 
+ l o c a l i z e d S t r i n g s [ " E x p i r e s " ]   =   " E x p i r e s " ; 
  l o c a l i z e d S t r i n g s [ " F i l e   s i z e " ]   =   " F i l e   s i z e " ; 
  l o c a l i z e d S t r i n g s [ " F o c u s   s e l e c t e d   f u n c t i o n . " ]   =   " F o c u s   o n   s e l e c t e d   f u n c t i o n . " ; 
  l o c a l i z e d S t r i n g s [ " F o n t s " ]   =   " F o n t s " ; 
+ l o c a l i z e d S t r i n g s [ " F o r m   D a t a " ]   =   " F o r m   D a t a " ; 
  l o c a l i z e d S t r i n g s [ " F u n c t i o n " ]   =   " F u n c t i o n " ; 
  l o c a l i z e d S t r i n g s [ " G R A P H S " ]   =   " G R A P H S " ; 
  l o c a l i z e d S t r i n g s [ " G l o b a l " ]   =   " G l o b a l " ; 
+ l o c a l i z e d S t r i n g s [ " H T T P " ]   =   " H T T P " ; 
  l o c a l i z e d S t r i n g s [ " H e a v y   ( B o t t o m   U p ) " ]   =   " H e a v y   ( B o t t o m   U p ) " ; 
+ l o c a l i z e d S t r i n g s [ " H i d e   c h a n g e s   v i e w . " ]   =   " H i d e   c h a n g e s   v i e w . " ; 
  l o c a l i z e d S t r i n g s [ " H i d e   c o n s o l e . " ]   =   " H i d e   c o n s o l e . " ; 
  l o c a l i z e d S t r i n g s [ " I m a g e s " ]   =   " I m a g e s " ; 
- l o c a l i z e d S t r i n g s [ " I n l i n e   S t y l e   A t t r i b u t e " ]   =   " I n l i n e   S t y l e   A t t r i b u t e " ; 
  l o c a l i z e d S t r i n g s [ " K e y " ]   =   " K e y " ; 
  l o c a l i z e d S t r i n g s [ " L O C A L   S T O R A G E " ]   =   " L O C A L   S T O R A G E " ; 
  l o c a l i z e d S t r i n g s [ " L o c a l " ]   =   " L o c a l " ; 
  l o c a l i z e d S t r i n g s [ " M I M E   t y p e " ]   =   " M I M E   t y p e " ; 
  l o c a l i z e d S t r i n g s [ " M e t r i c s " ]   =   " M e t r i c s " ; 
+ l o c a l i z e d S t r i n g s [ " N a m e " ]   =   " N a m e " ; 
+ l o c a l i z e d S t r i n g s [ " N e w   S t y l e   R u l e " ]   =   " N e w   S t y l e   R u l e " ; 
  l o c a l i z e d S t r i n g s [ " N o   B r e a k p o i n t s " ]   =   " N o   B r e a k p o i n t s " ; 
  l o c a l i z e d S t r i n g s [ " N o   P r o p e r t i e s " ]   =   " N o   P r o p e r t i e s " ; 
  l o c a l i z e d S t r i n g s [ " N o   V a r i a b l e s " ]   =   " N o   V a r i a b l e s " ; 
@@ -88,6 +99,7 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " N o t   P a u s e d " ]   =   " N o t   P a u s e d " ; 
  l o c a l i z e d S t r i n g s [ " O n l y   e n a b l e   f o r   t h i s   s e s s i o n " ]   =   " O n l y   e n a b l e   f o r   t h i s   s e s s i o n " ; 
  l o c a l i z e d S t r i n g s [ " O t h e r " ]   =   " O t h e r " ; 
+ l o c a l i z e d S t r i n g s [ " P a t h " ]   =   " P a t h " ; 
  l o c a l i z e d S t r i n g s [ " P a u s e   o n   e x c e p t i o n s . " ]   =   " P a u s e   o n   e x c e p t i o n s . " ; 
  l o c a l i z e d S t r i n g s [ " P a u s e   s c r i p t   e x e c u t i o n . " ]   =   " P a u s e   s c r i p t   e x e c u t i o n . " ; 
  l o c a l i z e d S t r i n g s [ " P a u s e d " ]   =   " P a u s e d " ; 
@@ -99,9 +111,11 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " P r o p e r t i e s " ]   =   " P r o p e r t i e s " ; 
  l o c a l i z e d S t r i n g s [ " P r o t o t y p e " ]   =   " P r o t o t y p e " ; 
  l o c a l i z e d S t r i n g s [ " Q u e r y " ]   =   " Q u e r y " ; 
+ l o c a l i z e d S t r i n g s [ " Q u e r y   S t r i n g   P a r a m e t e r s " ]   =   " Q u e r y   S t r i n g   P a r a m e t e r s " ; 
  l o c a l i z e d S t r i n g s [ " R E S O U R C E S " ]   =   " R E S O U R C E S " ; 
  l o c a l i z e d S t r i n g s [ " R e f r e s h " ]   =   " R e f r e s h " ; 
  l o c a l i z e d S t r i n g s [ " R e q u e s t   H e a d e r s " ]   =   " R e q u e s t   H e a d e r s " ; 
+ l o c a l i z e d S t r i n g s [ " R e q u e s t   P a y l o a d " ]   =   " R e q u e s t   P a y l o a d " ; 
  l o c a l i z e d S t r i n g s [ " R e s o u r c e   i n t e r p r e t e d   a s   % s   b u t   t r a n s f e r r e d   w i t h   M I M E   t y p e   % s . " ]   =   " R e s o u r c e   i n t e r p r e t e d   a s   % 1 $ s   b u t   t r a n s f e r r e d   w i t h   M I M E   t y p e   % 2 $ s . " ; 
  l o c a l i z e d S t r i n g s [ " R e s o u r c e   t r a c k i n g   d i s a b l e d .   C l i c k   t o   e n a b l e . " ]   =   " R e s o u r c e   t r a c k i n g   d i s a b l e d .   C l i c k   t o   e n a b l e . " ; 
  l o c a l i z e d S t r i n g s [ " R e s o u r c e   t r a c k i n g   e n a b l e d .   C l i c k   t o   d i s a b l e . " ]   =   " R e s o u r c e   t r a c k i n g   e n a b l e d .   C l i c k   t o   d i s a b l e . " ; 
@@ -113,9 +127,12 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " S c o p e   V a r i a b l e s " ]   =   " S c o p e   V a r i a b l e s " ; 
  l o c a l i z e d S t r i n g s [ " S c r i p t s " ]   =   " S c r i p t s " ; 
  l o c a l i z e d S t r i n g s [ " S e a r c h   % s " ]   =   " S e a r c h   % s " ; 
+ l o c a l i z e d S t r i n g s [ " S e c u r e " ]   =   " S e c u r e " ; 
  l o c a l i z e d S t r i n g s [ " S e l e c t   a n   e l e m e n t   i n   t h e   p a g e   t o   i n s p e c t   i t . " ]   =   " S e l e c t   a n   e l e m e n t   i n   t h e   p a g e   t o   i n s p e c t   i t . " ; 
  l o c a l i z e d S t r i n g s [ " S e l f " ]   =   " S e l f " ; 
+ l o c a l i z e d S t r i n g s [ " S e s s i o n " ]   =   " S e s s i o n " ; 
  l o c a l i z e d S t r i n g s [ " S h o w   a b s o l u t e   t o t a l   a n d   s e l f   t i m e s . " ]   =   " S h o w   a b s o l u t e   t o t a l   a n d   s e l f   t i m e s . " ; 
+ l o c a l i z e d S t r i n g s [ " S h o w   c h a n g e s   v i e w . " ]   =   " S h o w   c h a n g e s   v i e w . " ; 
  l o c a l i z e d S t r i n g s [ " S h o w   c o n s o l e . " ]   =   " S h o w   c o n s o l e . " ; 
  l o c a l i z e d S t r i n g s [ " S h o w   i n h e r i t e d " ]   =   " S h o w   i n h e r i t e d " ; 
  l o c a l i z e d S t r i n g s [ " S h o w   t h e   n e x t   s c r i p t   r e s o u r c e . " ]   =   " S h o w   t h e   n e x t   s c r i p t   r e s o u r c e . " ; 
@@ -135,6 +152,8 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " S t e p   o v e r   n e x t   f u n c t i o n   c a l l . " ]   =   " S t e p   o v e r   n e x t   f u n c t i o n   c a l l . " ; 
  l o c a l i z e d S t r i n g s [ " S t e p p i n g " ]   =   " S t e p p i n g " ; 
  l o c a l i z e d S t r i n g s [ " S t o p   p r o f i l i n g . " ]   =   " S t o p   p r o f i l i n g . " ; 
+ l o c a l i z e d S t r i n g s [ " S t o r a g e " ]   =   " S t o r a g e " ; 
+ l o c a l i z e d S t r i n g s [ " S t y l e   A t t r i b u t e " ]   =   " S t y l e   A t t r i b u t e " ; 
  l o c a l i z e d S t r i n g s [ " S t y l e s " ]   =   " S t y l e s " ; 
  l o c a l i z e d S t r i n g s [ " S t y l e s h e e t s " ]   =   " S t y l e s h e e t s " ; 
  l o c a l i z e d S t r i n g s [ " T h e    % s  \ n t a b l e   i s   e m p t y . " ]   =   " T h e    % s  \ n t a b l e   i s   e m p t y . " ; 
@@ -169,4 +188,16 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " s t y l e s h e e t " ]   =   " s t y l e s h e e t " ; 
  l o c a l i z e d S t r i n g s [ " u s e r   a g e n t   s t y l e s h e e t " ]   =   " u s e r   a g e n t   s t y l e s h e e t " ; 
  l o c a l i z e d S t r i n g s [ " u s e r   s t y l e s h e e t " ]   =   " u s e r   s t y l e s h e e t " ; 
+ l o c a l i z e d S t r i n g s [ " v i a   i n s p e c t o r " ]   =   " v i a   i n s p e c t o r " ; 
+ l o c a l i z e d S t r i n g s [ " T h e   b r e a k p o i n t   o n   l i n e   % d   w i l l   s t o p   o n l y   i f   t h i s   e x p r e s s i o n   i s   t r u e : " ]   =   " T h e   b r e a k p o i n t   o n   l i n e   % d   w i l l   s t o p   o n l y   i f   t h i s   e x p r e s s i o n   i s   t r u e : " ; 
+ l o c a l i z e d S t r i n g s [ " C P U   P R O F I L E S " ]   =   " C P U   P R O F I L E S " ; 
+ l o c a l i z e d S t r i n g s [ " H E A P   S N A P S H O T S " ]   =   " H E A P   S N A P S H O T S " ; 
+ l o c a l i z e d S t r i n g s [ " T a k e   h e a p   s n a p s h o t . " ]   =   " T a k e   h e a p   s n a p s h o t . " ; 
+ l o c a l i z e d S t r i n g s [ " H e x   C o l o r s " ]   =   " H e x   C o l o r s " ; 
+ l o c a l i z e d S t r i n g s [ " R G B   C o l o r s " ]   =   " R G B   C o l o r s " ; 
+ l o c a l i z e d S t r i n g s [ " H S L   C o l o r s " ]   =   " H S L   C o l o r s " ; 
+ l o c a l i z e d S t r i n g s [ " C l i c k   t o   c h a n g e   c o l o r   f o r m a t " ]   =   " C l i c k   t o   c h a n g e   c o l o r   f o r m a t " ; 
+ l o c a l i z e d S t r i n g s [ " W a t c h   E x p r e s s i o n s " ]   =   " W a t c h   E x p r e s s i o n s " ; 
+ l o c a l i z e d S t r i n g s [ " A d d " ]   =   " A d d " ; 
+ l o c a l i z e d S t r i n g s [ " D e l e t e   w a t c h   e x p r e s s i o n . " ]   =   " D e l e t e   w a t c h   e x p r e s s i o n . " ; 
  
\ No newline at end of file
