@@ -36,6 +36,7 @@
 #include "Console.h"
 #include "DOMSelection.h"
 #include "DOMTimer.h"
+#include "PageTransitionEvent.h"
 #include "Document.h"
 #include "Element.h"
 #include "EventException.h"
@@ -51,6 +52,7 @@
 #include "History.h"
 #include "InspectorController.h"
 #include "Location.h"
+#include "Media.h"
 #include "MessageEvent.h"
 #include "Navigator.h"
 #include "Page.h"
@@ -79,6 +81,10 @@
 #include "DOMApplicationCache.h"
 #endif
 
+#if ENABLE(NOTIFICATIONS)
+#include "NotificationCenter.h"
+#endif
+
 using std::min;
 using std::max;
 
@@ -86,24 +92,20 @@ namespace WebCore {
 
 class PostMessageTimer : public TimerBase {
 public:
-    PostMessageTimer(DOMWindow* window, const String& message, const String& sourceOrigin, PassRefPtr<DOMWindow> source, PassOwnPtr<MessagePortChannel> channel, SecurityOrigin* targetOrigin)
+    PostMessageTimer(DOMWindow* window, const String& message, const String& sourceOrigin, PassRefPtr<DOMWindow> source, PassOwnPtr<MessagePortChannelArray> channels, SecurityOrigin* targetOrigin)
         : m_window(window)
         , m_message(message)
         , m_origin(sourceOrigin)
         , m_source(source)
-        , m_channel(channel)
+        , m_channels(channels)
         , m_targetOrigin(targetOrigin)
     {
     }
 
     PassRefPtr<MessageEvent> event(ScriptExecutionContext* context)
     {
-        RefPtr<MessagePort> messagePort;
-        if (m_channel) {
-            messagePort = MessagePort::create(*context);
-            messagePort->entangle(m_channel.release());
-        }
-        return MessageEvent::create(m_message, m_origin, "", m_source, messagePort.release());
+        OwnPtr<MessagePortArray> messagePorts = MessagePort::entanglePorts(*context, m_channels.release());
+        return MessageEvent::create(m_message, m_origin, "", m_source, messagePorts.release());
     }
     SecurityOrigin* targetOrigin() const { return m_targetOrigin.get(); }
 
@@ -117,7 +119,7 @@ private:
     String m_message;
     String m_origin;
     RefPtr<DOMWindow> m_source;
-    OwnPtr<MessagePortChannel> m_channel;
+    OwnPtr<MessagePortChannelArray> m_channels;
     RefPtr<SecurityOrigin> m_targetOrigin;
 };
 
@@ -259,6 +261,7 @@ void DOMWindow::dispatchAllPendingUnloadEvents()
         if (!listeners)
             continue;
         RegisteredEventListenerVector listenersCopy = *listeners;
+        window->dispatchPageTransitionEvent(EventNames().pagehideEvent, false);
         window->dispatchUnloadEvent(&listenersCopy);
     }
 
@@ -453,6 +456,10 @@ void DOMWindow::clear()
         m_applicationCache->disconnectFrame();
     m_applicationCache = 0;
 #endif
+
+#if ENABLE(NOTIFICATIONS)
+    m_notifications = 0;
+#endif
 }
 
 Screen* DOMWindow::screen() const
@@ -559,7 +566,9 @@ Storage* DOMWindow::sessionStorage() const
         return 0;
 
     RefPtr<StorageArea> storageArea = page->sessionStorage()->storageArea(document->securityOrigin());
+#if ENABLE(INSPECTOR)
     page->inspectorController()->didUseDOMStorage(storageArea.get(), false, m_frame);
+#endif
 
     m_sessionStorage = Storage::create(m_frame, storageArea.release());
     return m_sessionStorage.get();
@@ -584,7 +593,9 @@ Storage* DOMWindow::localStorage() const
     StorageNamespace* localStorage = page->group().localStorage();
     RefPtr<StorageArea> storageArea = localStorage ? localStorage->storageArea(document->securityOrigin()) : 0; 
     if (storageArea) {
+#if ENABLE(INSPECTOR)
         page->inspectorController()->didUseDOMStorage(storageArea.get(), true, m_frame);
+#endif
         m_localStorage = Storage::create(m_frame, storageArea.release());
     }
 
@@ -592,7 +603,37 @@ Storage* DOMWindow::localStorage() const
 }
 #endif
 
-void DOMWindow::postMessage(const String& message, MessagePort* messagePort, const String& targetOrigin, DOMWindow* source, ExceptionCode& ec)
+#if ENABLE(NOTIFICATIONS)
+NotificationCenter* DOMWindow::webkitNotifications() const
+{
+    if (m_notifications)
+        return m_notifications.get();
+
+    Document* document = this->document();
+    if (!document)
+        return 0;
+    
+    Page* page = document->page();
+    if (!page)
+        return 0;
+
+    NotificationPresenter* provider = page->chrome()->notificationPresenter();
+    if (provider) 
+        m_notifications = NotificationCenter::create(document, provider);    
+      
+    return m_notifications.get();
+}
+#endif
+
+void DOMWindow::postMessage(const String& message, MessagePort* port, const String& targetOrigin, DOMWindow* source, ExceptionCode& ec)
+{
+    MessagePortArray ports;
+    if (port)
+        ports.append(port);
+    postMessage(message, &ports, targetOrigin, source, ec);
+}
+
+void DOMWindow::postMessage(const String& message, const MessagePortArray* ports, const String& targetOrigin, DOMWindow* source, ExceptionCode& ec)
 {
     if (!m_frame)
         return;
@@ -608,9 +649,7 @@ void DOMWindow::postMessage(const String& message, MessagePort* messagePort, con
         }
     }
 
-    OwnPtr<MessagePortChannel> channel;
-    if (messagePort)
-        channel = messagePort->disentangle(ec);
+    OwnPtr<MessagePortChannelArray> channels = MessagePort::disentanglePorts(ports, ec);
     if (ec)
         return;
 
@@ -622,7 +661,7 @@ void DOMWindow::postMessage(const String& message, MessagePort* messagePort, con
     String sourceOrigin = sourceDocument->securityOrigin()->toString();
 
     // Schedule the message.
-    PostMessageTimer* timer = new PostMessageTimer(this, message, sourceOrigin, source, channel.release(), target.get());
+    PostMessageTimer* timer = new PostMessageTimer(this, message, sourceOrigin, source, channels.release(), target.get());
     timer->startOneShot(0);
 }
 
@@ -1002,6 +1041,11 @@ Document* DOMWindow::document() const
     return m_frame->document();
 }
 
+PassRefPtr<Media> DOMWindow::media() const
+{
+    return Media::create(const_cast<DOMWindow*>(this));
+}
+
 PassRefPtr<CSSStyleDeclaration> DOMWindow::getComputedStyle(Element* elt, const String&) const
 {
     if (!elt)
@@ -1239,7 +1283,7 @@ void DOMWindow::removeEventListener(const AtomicString& eventType, EventListener
     size_t size = m_eventListeners.size();
     for (size_t i = 0; i < size; ++i) {
         RegisteredEventListener& r = *m_eventListeners[i];
-        if (r.eventType() == eventType && r.listener() == listener && r.useCapture() == useCapture) {
+        if (r.eventType() == eventType && r.useCapture() == useCapture && *r.listener() == *listener) {
             if (eventType == eventNames().unloadEvent)
                 removePendingEventListener(pendingUnloadEventListenerMap(), this, &r);
             else if (eventType == eventNames().beforeunloadEvent && allowsPendingBeforeUnloadListeners(this))
@@ -1323,6 +1367,11 @@ PassRefPtr<BeforeUnloadEvent> DOMWindow::dispatchBeforeUnloadEvent(RegisteredEve
     return beforeUnloadEvent.release();
 }
 
+void DOMWindow::dispatchPageTransitionEvent(const AtomicString& eventType, bool persisted)
+{
+    dispatchEventWithDocumentAsTarget(PageTransitionEvent::create(eventType, persisted));
+}
+
 void DOMWindow::removeAllEventListeners()
 {
     size_t size = m_eventListeners.size();
@@ -1519,6 +1568,16 @@ void DOMWindow::setOnfocus(PassRefPtr<EventListener> eventListener)
     setAttributeEventListener(eventNames().focusEvent, eventListener);
 }
 
+EventListener* DOMWindow::onhashchange() const
+{
+    return getAttributeEventListener(eventNames().hashchangeEvent);
+}
+
+void DOMWindow::setOnhashchange(PassRefPtr<EventListener> eventListener)
+{
+    setAttributeEventListener(eventNames().hashchangeEvent, eventListener);
+}
+
 EventListener* DOMWindow::onkeydown() const
 {
     return getAttributeEventListener(eventNames().keydownEvent);
@@ -1639,6 +1698,26 @@ void DOMWindow::setOnonline(PassRefPtr<EventListener> eventListener)
     setAttributeEventListener(eventNames().onlineEvent, eventListener);
 }
 
+EventListener* DOMWindow::onpagehide() const
+{
+    return getAttributeEventListener(eventNames().pagehideEvent);
+}
+
+void DOMWindow::setOnpagehide(PassRefPtr<EventListener> eventListener)
+{
+    setAttributeEventListener(eventNames().pagehideEvent, eventListener);
+}
+
+EventListener* DOMWindow::onpageshow() const
+{
+    return getAttributeEventListener(eventNames().pageshowEvent);
+}
+
+void DOMWindow::setOnpageshow(PassRefPtr<EventListener> eventListener)
+{
+    setAttributeEventListener(eventNames().pageshowEvent, eventListener);
+}
+
 EventListener* DOMWindow::onreset() const
 {
     return getAttributeEventListener(eventNames().resetEvent);
@@ -1999,6 +2078,16 @@ void DOMWindow::setOncontextmenu(PassRefPtr<EventListener> eventListener)
     setAttributeEventListener(eventNames().contextmenuEvent, eventListener);
 }
 
+EventListener* DOMWindow::oninvalid() const
+{
+    return getAttributeEventListener(eventNames().invalidEvent);
+}
+
+void DOMWindow::setOninvalid(PassRefPtr<EventListener> eventListener)
+{
+    setAttributeEventListener(eventNames().invalidEvent, eventListener);
+}
+
 void DOMWindow::captureEvents()
 {
     // Not implemented.
