@@ -65,10 +65,17 @@ class wxWindow;
 typedef wxWindow* PlatformWidget;
 #endif
 
+#if PLATFORM(HAIKU)
+class BView;
+typedef BView* PlatformWidget;
+#endif
+
 #if PLATFORM(CHROMIUM)
 #include "PlatformWidget.h"
 #endif
 
+typedef PlatformWidget PlatformPageClient;
+
 #include "IntPoint.h"
 #include "IntRect.h"
 #include "IntSize.h"
