@@ -21,7 +21,7 @@
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
  * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #ifndef PluginPackage_H
@@ -44,7 +44,7 @@ namespace WebCore {
     public:
         ~PluginPackage();
         static PassRefPtr<PluginPackage> createPackage(const String& path, const time_t& lastModified);
-        
+
         const String& name() const { return m_name; }
         const String& description() const { return m_description; }
         const String& path() const { return m_path; }
@@ -62,6 +62,9 @@ namespace WebCore {
         void unload();
         void unloadWithoutShutdown();
 
+        bool isEnabled() const { return m_isEnabled; }
+        void setEnabled(bool);
+
         const NPPluginFuncs* pluginFuncs() const { return &m_pluginFuncs; }
         int compareFileVersion(const PlatformModuleVersion&) const;
         int compare(const PluginPackage&) const;
@@ -77,6 +80,7 @@ namespace WebCore {
         void determineModuleVersionFromDescription();
         void initializeBrowserFuncs();
 
+        bool m_isEnabled;
         bool m_isLoaded;
         int m_loadCount;
 
