@@ -93,7 +93,7 @@ int FixedTableLayout::calcWidthArray(int)
     Length grpWidth;
     while (child) {
         if (child->isTableCol()) {
-            RenderTableCol* col = static_cast<RenderTableCol*>(child);
+            RenderTableCol* col = toRenderTableCol(child);
             if (col->firstChild())
                 grpWidth = col->style()->width();
             else {
@@ -128,7 +128,7 @@ int FixedTableLayout::calcWidthArray(int)
                     currentEffectiveColumn++;
                 }
             }
-            static_cast<RenderTableCol*>(child)->calcPrefWidths();
+            toRenderTableCol(child)->calcPrefWidths();
         } else
             break;
 
@@ -156,7 +156,7 @@ int FixedTableLayout::calcWidthArray(int)
         child = firstRow->firstChild();
         while (child) {
             if (child->isTableCell()) {
-                RenderTableCell* cell = static_cast<RenderTableCell*>(child);
+                RenderTableCell* cell = toRenderTableCell(child);
                 if (cell->prefWidthsDirty())
                     cell->calcPrefWidths();
 
